// Import specific D3 modules for use
import { select, selectAll, scaleOrdinal } from 'd3';

// Initialize SVG canvas and define dimensions and color scale
const svg = select('svg');
const width = +svg.attr('width');
const height = +svg.attr('height');
const centre_x = width / 2;
const centre_y = height / 2;
const colorScale = ['green'];

// Set the SVG's viewbox and preserveAspectRatio for responsive design
svg
  .attr('preserveAspectRatio', 'xMidYMid meet')
  .attr('viewBox', [0, 0, width * 2, height * 2]);

// Define markers used for arrows in the visualization
const defs = svg.append('defs');
defs
  .selectAll('marker')
  .data(['node'])
  .enter()
  .append('marker')
  .attr('id', (d) => d)
  .attr('viewBox', '0 -2.5 5 5')
  .attr('refX', 25)
  .attr('refY', 0)
  .attr('markerWidth', 5)
  .attr('markerHeight', 9)
  .attr('orient', 'auto')
  .append('path')
  .attr('d', 'M0,-2.5L5,0L0,2.5')
  .attr('fill', '#999');

// Load data from a JSON file and process it
d3.json('data.json').then((data) => {
  // Process link data for connections between nodes
  const links = data.links.map((d) => Object.create(d));
  links.forEach((d, i) => {
    d.srcType = data.links[i].srcType;
    d.tgtType = data.links[i].tgtType;
    d.relationship = data.links[i].relationship;
    d.value = data.links[i].value;
  });

  // Process node data for visualization elements
  const nodes = data.nodes.map((d) => Object.create(d));
  nodes.forEach((d, i) => {
    d.group = data.nodes[i].group;
  });

  // Initialize force simulation for dynamic node positioning
  const simulation = d3
    .forceSimulation(nodes)
    .force(
      'link',
      d3
        .forceLink(links)
        .id((d) => d.id)
        .distance(120),
    )
    .force('charge', d3.forceManyBody().strength(-500))
    .force('collision', d3.forceCollide().radius(105))
    .force('center', d3.forceCenter(width, height));

  // Define drag behavior for nodes
  const drag = (simulation) => {
    function dragstarted(event) {
      if (!event.active)
        simulation.alphaTarget(0.5).restart();
      event.subject.fx = event.subject.x;
      event.subject.fy = event.subject.y;
    }

    function dragged(event) {
      event.subject.fx = event.x;
      event.subject.fy = event.y;
    }

    function dragended(event, d) {
      if (!event.active) simulation.alphaTarget(0);
      event.subject.fx = null;
      event.subject.fy = null;
    }
    return d3
      .drag()
      .on('start', dragstarted)
      .on('drag', dragged)
      .on('end', dragended);
  };

  // Create link elements for connections between nodes
  const link = svg
    .append('g')
    .selectAll('line')
    .data(links)
    .join('line')
    .attr('marker-end', 'url(#node)');

  // Create and position node elements
  const node = svg
    .append('g')
    .attr('class', 'node')
    .selectAll('g')
    .data(nodes)
    .enter()
    .append('g')
    .attr(
      'transform',
      (d) =>
        'translate(' + centre_x + ', ' + centre_y + ')',
    );

  // Add circles to represent nodes
  const circles = node
    .append('circle')
    .attr('r', 75)
    .attr('fill', (d) => colorScale[0])
    .call(drag(simulation));

  // Add text labels to nodes
  const nodeLbl = node
    .append('text')
    .text((d) => d.id)
    .attr('dy', '0.3em');

  // Update position of nodes and links on simulation tick
  simulation.on('tick', () => {
    link
      .attr('x1', (d) => d.source.x)
      .attr('y1', (d) => d.source.y)
      .attr('x2', (d) => d.target.x)
      .attr('y2', (d) => d.target.y);
    node.attr(
      'transform',
      (d) => 'translate(' + d.x + ', ' + d.y + ')',
    );
  });
});
